@@ -440,6 +440,22 @@ bool DecodeArgs(int argc, char ** argv)
 	    option_flags &= ~NO_GSSAFER;
 	    Message(PARSE_STDIN,"GhostScript calls use -dSAFER\n");
 	  }
+	} else if (strncmp(p,"ogs",3)==0) { 
+	  if (p[3] != '0') {
+	    option_flags |= NO_GHOSTSCRIPT;
+	    Message(PARSE_STDIN,"No GhostScript calls\n",p);
+	  } else {
+	    option_flags &= ~NO_GHOSTSCRIPT;
+	    Message(PARSE_STDIN,"GhostScript calls made\n");
+	  }
+	} else if (strncmp(p,"orawps",6)==0) { 
+	  if (p[6] != '0') {
+	    option_flags |= NO_RAW_PS;
+	    Message(PARSE_STDIN,"Conversion of raw PostScript will not be attempted\n",p);
+	  } else {
+	    option_flags &= ~NO_RAW_PS;
+	    Message(PARSE_STDIN,"Conversion of raw PostScript will be attempted\n",p);
+	  }
 	} else
 	  goto DEFAULT;
 	break ;
@@ -604,15 +620,16 @@ or <http://www.gnu.org/licenses/>.");
     fprintf(stdout,"  -fg s        Foreground color (TeX-style color)\n");
     fprintf(stdout,"  --follow*    Wait for data at end-of-file\n");
 #ifdef HAVE_FT2
-    fprintf(stdout,"  --freetype*  FreeType font rendering (default on)\n");
+    fprintf(stdout,"  --freetype*  FreeType font rendering (preferred, default on)\n");
 #endif
     fprintf(stdout,"  --gamma #    Control color interpolation\n");
 #ifdef HAVE_GDIMAGEGIF
     fprintf(stdout,"  --gif        Output GIF images (dvigif default)\n");
 #endif
     fprintf(stdout,"  --height*    Output the image height on stdout\n");
-    fprintf(stdout,"  --noghostscript*  Don't use ghostscript for PostScript specials\n");
+    fprintf(stdout,"  --nogs*      Don't use ghostscript for PostScript specials\n");
     fprintf(stdout,"  --nogssafer* Don't use -dSAFER in ghostscript calls\n");
+    fprintf(stdout,"  --norawps*   Don't convert raw PostScript specials\n");
 #ifdef HAVE_GDIMAGECREATETRUECOLOR
     fprintf(stdout,"  --palette*   Force palette output\n");
 #endif
@@ -628,6 +645,7 @@ or <http://www.gnu.org/licenses/>.");
     fprintf(stdout,"  --truecolor* Truecolor output\n");
 #endif
     fprintf(stdout,"  -Q #         Quality (T1lib and PK subsampling)\n");
+    fprintf(stdout,"  --width*     Output the image width on stdout\n");
 #ifdef HAVE_GDIMAGEPNGEX
     fprintf(stdout,"  -z #         PNG compression level\n");
 #endif
@@ -657,13 +675,14 @@ void DecodeString(char *string)
   int     strc=1;
   strv[0]=NULL;                       /* No program name */
 
-  while (*string==' ' || *string=='\t' || *string=='\n') 
+  while (*string==' ' || *string=='\t' || *string=='\r' || *string=='\n') 
     string++;
   while (*string!='\0') {
     strv[strc++]=string;
     if (*string!='\'') {
       /* Normal split at whitespace */
-      while (*string!=' ' && *string!='\t' && *string!='\n' && *string!='\0') 
+      while (*string!=' ' && *string!='\t' \
+	     && *string!='\n' && *string!='\r' && *string!='\0')
 	string++;
     } else {
       /* String delimiter found , so search for next */
@@ -673,7 +692,7 @@ void DecodeString(char *string)
     }
     if (*string!='\0')
       *string++='\0';
-    while (*string==' ' || *string=='\t' || *string=='\n') 
+    while (*string==' ' || *string=='\t' || *string=='\r' || *string=='\n') 
       string++;
   }
   if (strc>1) /* Nonempty */
