@@ -18,14 +18,11 @@
   License along with this program. If not, see
   <http://www.gnu.org/licenses/>.
 
-  Copyright (C) 2002-2008 Jan-ke Larsson
+  Copyright (C) 2002-2009 Jan-ke Larsson
 
 ************************************************************************/
 
 #include "dvipng.h"
-#if HAVE_ALLOCA_H
-# include <alloca.h>
-#endif
 
 void LoadT1(int32_t c, struct char_entry * ptr)
 {
@@ -100,7 +97,7 @@ void LoadT1(int32_t c, struct char_entry * ptr)
     Shrink raster while doing antialiasing. 
   */
   if ((ptr->data = calloc(shrunk_width*shrunk_height,sizeof(char))) == NULL)
-    Fatal("unable to allocate image space for char %c", (char)c);
+    Fatal("unable to malloc image space for char %c", (char)c);
   for (j = 0; j < original_height; j++) {	
     for (i = 0; i < (original_width+7)/8 ; i++) {    
       for (k = 0; k < 8 ; k++) {
@@ -164,7 +161,7 @@ bool InitT1(struct font_entry * tfontp)
 }
 
 
-void UnLoadT1(struct char_entry *ptr)
+static void UnLoadT1(struct char_entry *ptr)
 {
   if (ptr->data!=NULL)
     free(ptr->data);
@@ -187,7 +184,9 @@ void DoneT1(struct font_entry *tfontp)
     }
     c++;
   }
-  tfontp->name[0]='\0';
+  if (tfontp->name!=NULL)
+    free(tfontp->name);
+  tfontp->name=NULL;
 }
 
 
